# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import core_v1
from . import runtime
from . import meta_v1
from . import resource


@dataclass
class AllocationResult(DictMixin):
    """AllocationResult contains attributes of an allocated resource.

      **parameters**

      * **availableOnNodes** ``Optional[core_v1.NodeSelector]`` - This field will get set by the resource driver after it has allocated the
        resource to inform the scheduler where it can schedule Pods using the
        ResourceClaim.
        Setting this field is optional. If null, the resource is available everywhere.
      * **resourceHandles** ``Optional[List[ResourceHandle]]`` - ResourceHandles contain the state associated with an allocation that should be
        maintained throughout the lifetime of a claim. Each ResourceHandle contains
        data that should be passed to a specific kubelet plugin once it lands on a
        node. This data is returned by the driver after a successful allocation and is
        opaque to Kubernetes. Driver documentation may explain to users how to
        interpret this data if needed.
        Setting this field is optional. It has a maximum size of 32 entries. If null
        (or empty), it is assumed this allocation will be processed by a single
        kubelet plugin with no ResourceHandle data attached. The name of the kubelet
        plugin invoked will match the DriverName set in the ResourceClaimStatus this
        AllocationResult is embedded in.
      * **shareable** ``Optional[bool]`` - Shareable determines whether the resource supports more than one consumer at a
        time.
    """
    availableOnNodes: 'Optional[core_v1.NodeSelector]' = None
    resourceHandles: 'Optional[List[ResourceHandle]]' = None
    shareable: 'Optional[bool]' = None


@dataclass
class DriverAllocationResult(DictMixin):
    """DriverAllocationResult contains vendor parameters and the allocation result
      for one request.

      **parameters**

      * **namedResources** ``Optional[NamedResourcesAllocationResult]`` - NamedResources describes the allocation result when using the named resources
        model.
      * **vendorRequestParameters** ``Optional[runtime.RawExtension]`` - VendorRequestParameters are the per-request configuration parameters from the
        time that the claim was allocated.
    """
    namedResources: 'Optional[NamedResourcesAllocationResult]' = None
    vendorRequestParameters: 'Optional[runtime.RawExtension]' = None


@dataclass
class DriverRequests(DictMixin):
    """DriverRequests describes all resources that are needed from one particular
      driver.

      **parameters**

      * **driverName** ``Optional[str]`` - DriverName is the name used by the DRA driver kubelet plugin.
      * **requests** ``Optional[List[ResourceRequest]]`` - Requests describes all resources that are needed from the driver.
      * **vendorParameters** ``Optional[runtime.RawExtension]`` - VendorParameters are arbitrary setup parameters for all requests of the claim.
        They are ignored while allocating the claim.
    """
    driverName: 'Optional[str]' = None
    requests: 'Optional[List[ResourceRequest]]' = None
    vendorParameters: 'Optional[runtime.RawExtension]' = None


@dataclass
class NamedResourcesAllocationResult(DictMixin):
    """NamedResourcesAllocationResult is used in AllocationResultModel.

      **parameters**

      * **name** ``str`` - Name is the name of the selected resource instance.
    """
    name: 'str'


@dataclass
class NamedResourcesAttribute(DictMixin):
    """NamedResourcesAttribute is a combination of an attribute name and its value.

      **parameters**

      * **name** ``str`` - Name is unique identifier among all resource instances managed by the driver
        on the node. It must be a DNS subdomain.
      * **bool** ``Optional[bool]`` - BoolValue is a true/false value.
      * **int** ``Optional[int]`` - IntValue is a 64-bit integer.
      * **intSlice** ``Optional[NamedResourcesIntSlice]`` - IntSliceValue is an array of 64-bit integers.
      * **quantity** ``Optional[resource.Quantity]`` - QuantityValue is a quantity.
      * **string** ``Optional[str]`` - StringValue is a string.
      * **stringSlice** ``Optional[NamedResourcesStringSlice]`` - StringSliceValue is an array of strings.
      * **version** ``Optional[str]`` - VersionValue is a semantic version according to semver.org spec 2.0.0.
    """
    name: 'str'
    bool: 'Optional[bool]' = None
    int: 'Optional[int]' = None
    intSlice: 'Optional[NamedResourcesIntSlice]' = None
    quantity: 'Optional[resource.Quantity]' = None
    string: 'Optional[str]' = None
    stringSlice: 'Optional[NamedResourcesStringSlice]' = None
    version: 'Optional[str]' = None


@dataclass
class NamedResourcesFilter(DictMixin):
    """NamedResourcesFilter is used in ResourceFilterModel.

      **parameters**

      * **selector** ``str`` - Selector is a CEL expression which must evaluate to true if a resource
        instance is suitable. The language is as defined in
        https://kubernetes.io/docs/reference/using-api/cel/
        In addition, for each type NamedResourcesin AttributeValue there is a map that
        resolves to the corresponding value of the instance under evaluation. For
        example:
           attributes.quantity["a"].isGreaterThan(quantity("0")) &&
           attributes.stringslice["b"].isSorted()
    """
    selector: 'str'


@dataclass
class NamedResourcesInstance(DictMixin):
    """NamedResourcesInstance represents one individual hardware instance that can be
      selected based on its attributes.

      **parameters**

      * **name** ``str`` - Name is unique identifier among all resource instances managed by the driver
        on the node. It must be a DNS subdomain.
      * **attributes** ``Optional[List[NamedResourcesAttribute]]`` - Attributes defines the attributes of this resource instance. The name of each
        attribute must be unique.
    """
    name: 'str'
    attributes: 'Optional[List[NamedResourcesAttribute]]' = None


@dataclass
class NamedResourcesIntSlice(DictMixin):
    """NamedResourcesIntSlice contains a slice of 64-bit integers.

      **parameters**

      * **ints** ``List[int]`` - Ints is the slice of 64-bit integers.
    """
    ints: 'List[int]'


@dataclass
class NamedResourcesRequest(DictMixin):
    """NamedResourcesRequest is used in ResourceRequestModel.

      **parameters**

      * **selector** ``str`` - Selector is a CEL expression which must evaluate to true if a resource
        instance is suitable. The language is as defined in
        https://kubernetes.io/docs/reference/using-api/cel/
        In addition, for each type NamedResourcesin AttributeValue there is a map that
        resolves to the corresponding value of the instance under evaluation. For
        example:
           attributes.quantity["a"].isGreaterThan(quantity("0")) &&
           attributes.stringslice["b"].isSorted()
    """
    selector: 'str'


@dataclass
class NamedResourcesResources(DictMixin):
    """NamedResourcesResources is used in ResourceModel.

      **parameters**

      * **instances** ``List[NamedResourcesInstance]`` - The list of all individual resources instances currently available.
    """
    instances: 'List[NamedResourcesInstance]'


@dataclass
class NamedResourcesStringSlice(DictMixin):
    """NamedResourcesStringSlice contains a slice of strings.

      **parameters**

      * **strings** ``List[str]`` - Strings is the slice of strings.
    """
    strings: 'List[str]'


@dataclass
class PodSchedulingContext(DictMixin):
    """PodSchedulingContext objects hold information that is needed to schedule a Pod
      with ResourceClaims that use "WaitForFirstConsumer" allocation mode.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **spec** ``PodSchedulingContextSpec`` - Spec describes where resources for the Pod are needed.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **status** ``Optional[PodSchedulingContextStatus]`` - Status describes where resources for the Pod can be allocated.
    """
    spec: 'PodSchedulingContextSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[PodSchedulingContextStatus]' = None


@dataclass
class PodSchedulingContextList(DictMixin):
    """PodSchedulingContextList is a collection of Pod scheduling objects.

      **parameters**

      * **items** ``List[PodSchedulingContext]`` - Items is the list of PodSchedulingContext objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[PodSchedulingContext]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class PodSchedulingContextSpec(DictMixin):
    """PodSchedulingContextSpec describes where resources for the Pod are needed.

      **parameters**

      * **potentialNodes** ``Optional[List[str]]`` - PotentialNodes lists nodes where the Pod might be able to run.
        The size of this field is limited to 128. This is large enough for many
        clusters. Larger clusters may need more attempts to find a node that suits all
        pending resources. This may get increased in the future, but not reduced.
      * **selectedNode** ``Optional[str]`` - SelectedNode is the node for which allocation of ResourceClaims that are
        referenced by the Pod and that use "WaitForFirstConsumer" allocation is to be
        attempted.
    """
    potentialNodes: 'Optional[List[str]]' = None
    selectedNode: 'Optional[str]' = None


@dataclass
class PodSchedulingContextStatus(DictMixin):
    """PodSchedulingContextStatus describes where resources for the Pod can be
      allocated.

      **parameters**

      * **resourceClaims** ``Optional[List[ResourceClaimSchedulingStatus]]`` - ResourceClaims describes resource availability for each pod.spec.resourceClaim
        entry where the corresponding ResourceClaim uses "WaitForFirstConsumer"
        allocation mode.
    """
    resourceClaims: 'Optional[List[ResourceClaimSchedulingStatus]]' = None


@dataclass
class ResourceClaim(DictMixin):
    """ResourceClaim describes which resources are needed by a resource consumer. Its
      status tracks whether the resource has been allocated and what the resulting
      attributes are.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **spec** ``ResourceClaimSpec`` - Spec describes the desired attributes of a resource that then needs to be
        allocated. It can only be set once when creating the ResourceClaim.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **status** ``Optional[ResourceClaimStatus]`` - Status describes whether the resource is available and with which attributes.
    """
    spec: 'ResourceClaimSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[ResourceClaimStatus]' = None


@dataclass
class ResourceClaimConsumerReference(DictMixin):
    """ResourceClaimConsumerReference contains enough information to let you locate
      the consumer of a ResourceClaim. The user must be a resource in the same
      namespace as the ResourceClaim.

      **parameters**

      * **name** ``str`` - Name is the name of resource being referenced.
      * **resource** ``str`` - Resource is the type of resource being referenced, for example "pods".
      * **uid** ``str`` - UID identifies exactly one incarnation of the resource.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
    """
    name: 'str'
    resource: 'str'
    uid: 'str'
    apiGroup: 'Optional[str]' = None


@dataclass
class ResourceClaimList(DictMixin):
    """ResourceClaimList is a collection of claims.

      **parameters**

      * **items** ``List[ResourceClaim]`` - Items is the list of resource claims.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClaim]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClaimParameters(DictMixin):
    """ResourceClaimParameters defines resource requests for a ResourceClaim in an
      in-tree format understood by Kubernetes.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **driverRequests** ``Optional[List[DriverRequests]]`` - DriverRequests describes all resources that are needed for the allocated
        claim. A single claim may use resources coming from different drivers. For
        each driver, this array has at most one entry which then may have one or more
        per-driver requests.
        May be empty, in which case the claim can always be allocated.
      * **generatedFrom** ``Optional[ResourceClaimParametersReference]`` - If this object was created from some other resource, then this links back to
        that resource. This field is used to find the in-tree representation of the
        claim parameters when the parameter reference of the claim refers to some
        unknown type.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **shareable** ``Optional[bool]`` - Shareable indicates whether the allocated claim is meant to be shareable by
        multiple consumers at the same time.
    """
    apiVersion: 'Optional[str]' = None
    driverRequests: 'Optional[List[DriverRequests]]' = None
    generatedFrom: 'Optional[ResourceClaimParametersReference]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    shareable: 'Optional[bool]' = None


@dataclass
class ResourceClaimParametersList(DictMixin):
    """ResourceClaimParametersList is a collection of ResourceClaimParameters.

      **parameters**

      * **items** ``List[ResourceClaimParameters]`` - Items is the list of node resource capacity objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClaimParameters]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClaimParametersReference(DictMixin):
    """ResourceClaimParametersReference contains enough information to let you locate
      the parameters for a ResourceClaim. The object must be in the same namespace
      as the ResourceClaim.

      **parameters**

      * **kind** ``str`` - Kind is the type of resource being referenced. This is the same value as in
        the parameter object's metadata, for example "ConfigMap".
      * **name** ``str`` - Name is the name of resource being referenced.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
    """
    kind: 'str'
    name: 'str'
    apiGroup: 'Optional[str]' = None


@dataclass
class ResourceClaimSchedulingStatus(DictMixin):
    """ResourceClaimSchedulingStatus contains information about one particular
      ResourceClaim with "WaitForFirstConsumer" allocation mode.

      **parameters**

      * **name** ``Optional[str]`` - Name matches the pod.spec.resourceClaims[*].Name field.
      * **unsuitableNodes** ``Optional[List[str]]`` - UnsuitableNodes lists nodes that the ResourceClaim cannot be allocated for.
        The size of this field is limited to 128, the same as for
        PodSchedulingSpec.PotentialNodes. This may get increased in the future, but
        not reduced.
    """
    name: 'Optional[str]' = None
    unsuitableNodes: 'Optional[List[str]]' = None


@dataclass
class ResourceClaimSpec(DictMixin):
    """ResourceClaimSpec defines how a resource is to be allocated.

      **parameters**

      * **resourceClassName** ``str`` - ResourceClassName references the driver and additional parameters via the name
        of a ResourceClass that was created as part of the driver deployment.
      * **allocationMode** ``Optional[str]`` - Allocation can start immediately or when a Pod wants to use the resource.
        "WaitForFirstConsumer" is the default.
      * **parametersRef** ``Optional[ResourceClaimParametersReference]`` - ParametersRef references a separate object with arbitrary parameters that will
        be used by the driver when allocating a resource for the claim.
        The object must be in the same namespace as the ResourceClaim.
    """
    resourceClassName: 'str'
    allocationMode: 'Optional[str]' = None
    parametersRef: 'Optional[ResourceClaimParametersReference]' = None


@dataclass
class ResourceClaimStatus(DictMixin):
    """ResourceClaimStatus tracks whether the resource has been allocated and what
      the resulting attributes are.

      **parameters**

      * **allocation** ``Optional[AllocationResult]`` - Allocation is set by the resource driver once a resource or set of resources
        has been allocated successfully. If this is not specified, the resources have
        not been allocated yet.
      * **deallocationRequested** ``Optional[bool]`` - DeallocationRequested indicates that a ResourceClaim is to be deallocated.
        The driver then must deallocate this claim and reset the field together with
        clearing the Allocation field.
        While DeallocationRequested is set, no new consumers may be added to
        ReservedFor.
      * **driverName** ``Optional[str]`` - DriverName is a copy of the driver name from the ResourceClass at the time
        when allocation started.
      * **reservedFor** ``Optional[List[ResourceClaimConsumerReference]]`` - ReservedFor indicates which entities are currently allowed to use the claim. A
        Pod which references a ResourceClaim which is not reserved for that Pod will
        not be started.
        There can be at most 32 such reservations. This may get increased in the
        future, but not reduced.
    """
    allocation: 'Optional[AllocationResult]' = None
    deallocationRequested: 'Optional[bool]' = None
    driverName: 'Optional[str]' = None
    reservedFor: 'Optional[List[ResourceClaimConsumerReference]]' = None


@dataclass
class ResourceClaimTemplate(DictMixin):
    """ResourceClaimTemplate is used to produce ResourceClaim objects.

      **parameters**

      * **spec** ``ResourceClaimTemplateSpec`` - Describes the ResourceClaim that is to be generated.
        This field is immutable. A ResourceClaim will get created by the control plane
        for a Pod when needed and then not get updated anymore.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
    """
    spec: 'ResourceClaimTemplateSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class ResourceClaimTemplateList(DictMixin):
    """ResourceClaimTemplateList is a collection of claim templates.

      **parameters**

      * **items** ``List[ResourceClaimTemplate]`` - Items is the list of resource claim templates.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClaimTemplate]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClaimTemplateSpec(DictMixin):
    """ResourceClaimTemplateSpec contains the metadata and fields for a
      ResourceClaim.

      **parameters**

      * **spec** ``ResourceClaimSpec`` - Spec for the ResourceClaim. The entire content is copied unchanged into the
        ResourceClaim that gets created from this template. The same fields as in a
        ResourceClaim are also valid here.
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - ObjectMeta may contain labels and annotations that will be copied into the PVC
        when creating it. No other fields are allowed and will be rejected during
        validation.
    """
    spec: 'ResourceClaimSpec'
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class ResourceClass(DictMixin):
    """ResourceClass is used by administrators to influence how resources are
      allocated.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **driverName** ``str`` - DriverName defines the name of the dynamic resource driver that is used for
        allocation of a ResourceClaim that uses this class.
        Resource drivers have a unique name in forward domain order
        (acme.example.com).
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **parametersRef** ``Optional[ResourceClassParametersReference]`` - ParametersRef references an arbitrary separate object that may hold parameters
        that will be used by the driver when allocating a resource that uses this
        class. A dynamic resource driver can distinguish between parameters stored
        here and and those stored in ResourceClaimSpec.
      * **structuredParameters** ``Optional[bool]`` - If and only if allocation of claims using this class is handled via structured
        parameters, then StructuredParameters must be set to true.
      * **suitableNodes** ``Optional[core_v1.NodeSelector]`` - Only nodes matching the selector will be considered by the scheduler when
        trying to find a Node that fits a Pod when that Pod uses a ResourceClaim that
        has not been allocated yet.
        Setting this field is optional. If null, all nodes are candidates.
    """
    driverName: 'str'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    parametersRef: 'Optional[ResourceClassParametersReference]' = None
    structuredParameters: 'Optional[bool]' = None
    suitableNodes: 'Optional[core_v1.NodeSelector]' = None


@dataclass
class ResourceClassList(DictMixin):
    """ResourceClassList is a collection of classes.

      **parameters**

      * **items** ``List[ResourceClass]`` - Items is the list of resource classes.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClass]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClassParameters(DictMixin):
    """ResourceClassParameters defines resource requests for a ResourceClass in an
      in-tree format understood by Kubernetes.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **filters** ``Optional[List[ResourceFilter]]`` - Filters describes additional contraints that must be met when using the class.
      * **generatedFrom** ``Optional[ResourceClassParametersReference]`` - If this object was created from some other resource, then this links back to
        that resource. This field is used to find the in-tree representation of the
        class parameters when the parameter reference of the class refers to some
        unknown type.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **vendorParameters** ``Optional[List[VendorParameters]]`` - VendorParameters are arbitrary setup parameters for all claims using this
        class. They are ignored while allocating the claim. There must not be more
        than one entry per driver.
    """
    apiVersion: 'Optional[str]' = None
    filters: 'Optional[List[ResourceFilter]]' = None
    generatedFrom: 'Optional[ResourceClassParametersReference]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    vendorParameters: 'Optional[List[VendorParameters]]' = None


@dataclass
class ResourceClassParametersList(DictMixin):
    """ResourceClassParametersList is a collection of ResourceClassParameters.

      **parameters**

      * **items** ``List[ResourceClassParameters]`` - Items is the list of node resource capacity objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClassParameters]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClassParametersReference(DictMixin):
    """ResourceClassParametersReference contains enough information to let you locate
      the parameters for a ResourceClass.

      **parameters**

      * **kind** ``str`` - Kind is the type of resource being referenced. This is the same value as in
        the parameter object's metadata.
      * **name** ``str`` - Name is the name of resource being referenced.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
      * **namespace** ``Optional[str]`` - Namespace that contains the referenced resource. Must be empty for
        cluster-scoped resources and non-empty for namespaced resources.
    """
    kind: 'str'
    name: 'str'
    apiGroup: 'Optional[str]' = None
    namespace: 'Optional[str]' = None


@dataclass
class ResourceFilter(DictMixin):
    """ResourceFilter is a filter for resources from one particular driver.

      **parameters**

      * **driverName** ``Optional[str]`` - DriverName is the name used by the DRA driver kubelet plugin.
      * **namedResources** ``Optional[NamedResourcesFilter]`` - NamedResources describes a resource filter using the named resources model.
    """
    driverName: 'Optional[str]' = None
    namedResources: 'Optional[NamedResourcesFilter]' = None


@dataclass
class ResourceHandle(DictMixin):
    """ResourceHandle holds opaque resource data for processing by a specific kubelet
      plugin.

      **parameters**

      * **data** ``Optional[str]`` - Data contains the opaque data associated with this ResourceHandle. It is set
        by the controller component of the resource driver whose name matches the
        DriverName set in the ResourceClaimStatus this ResourceHandle is embedded in.
        It is set at allocation time and is intended for processing by the kubelet
        plugin whose name matches the DriverName set in this ResourceHandle.
        The maximum size of this field is 16KiB. This may get increased in the future,
        but not reduced.
      * **driverName** ``Optional[str]`` - DriverName specifies the name of the resource driver whose kubelet plugin
        should be invoked to process this ResourceHandle's data once it lands on a
        node. This may differ from the DriverName set in ResourceClaimStatus this
        ResourceHandle is embedded in.
      * **structuredData** ``Optional[StructuredResourceHandle]`` - If StructuredData is set, then it needs to be used instead of Data.
    """
    data: 'Optional[str]' = None
    driverName: 'Optional[str]' = None
    structuredData: 'Optional[StructuredResourceHandle]' = None


@dataclass
class ResourceRequest(DictMixin):
    """ResourceRequest is a request for resources from one particular driver.

      **parameters**

      * **namedResources** ``Optional[NamedResourcesRequest]`` - NamedResources describes a request for resources with the named resources
        model.
      * **vendorParameters** ``Optional[runtime.RawExtension]`` - VendorParameters are arbitrary setup parameters for the requested resource.
        They are ignored while allocating a claim.
    """
    namedResources: 'Optional[NamedResourcesRequest]' = None
    vendorParameters: 'Optional[runtime.RawExtension]' = None


@dataclass
class ResourceSlice(DictMixin):
    """ResourceSlice provides information about available resources on individual
      nodes.

      **parameters**

      * **driverName** ``str`` - DriverName identifies the DRA driver providing the capacity information. A
        field selector can be used to list only ResourceSlice objects with a certain
        driver name.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **namedResources** ``Optional[NamedResourcesResources]`` - NamedResources describes available resources using the named resources model.
      * **nodeName** ``Optional[str]`` - NodeName identifies the node which provides the resources if they are local to
        a node.
        A field selector can be used to list only ResourceSlice objects with a certain
        node name.
    """
    driverName: 'str'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    namedResources: 'Optional[NamedResourcesResources]' = None
    nodeName: 'Optional[str]' = None


@dataclass
class ResourceSliceList(DictMixin):
    """ResourceSliceList is a collection of ResourceSlices.

      **parameters**

      * **items** ``List[ResourceSlice]`` - Items is the list of node resource capacity objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceSlice]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class StructuredResourceHandle(DictMixin):
    """StructuredResourceHandle is the in-tree representation of the allocation
      result.

      **parameters**

      * **results** ``List[DriverAllocationResult]`` - Results lists all allocated driver resources.
      * **nodeName** ``Optional[str]`` - NodeName is the name of the node providing the necessary resources if the
        resources are local to a node.
      * **vendorClaimParameters** ``Optional[runtime.RawExtension]`` - VendorClaimParameters are the per-claim configuration parameters from the
        resource claim parameters at the time that the claim was allocated.
      * **vendorClassParameters** ``Optional[runtime.RawExtension]`` - VendorClassParameters are the per-claim configuration parameters from the
        resource class at the time that the claim was allocated.
    """
    results: 'List[DriverAllocationResult]'
    nodeName: 'Optional[str]' = None
    vendorClaimParameters: 'Optional[runtime.RawExtension]' = None
    vendorClassParameters: 'Optional[runtime.RawExtension]' = None


@dataclass
class VendorParameters(DictMixin):
    """VendorParameters are opaque parameters for one particular driver.

      **parameters**

      * **driverName** ``Optional[str]`` - DriverName is the name used by the DRA driver kubelet plugin.
      * **parameters** ``Optional[runtime.RawExtension]`` - Parameters can be arbitrary setup parameters. They are ignored while
        allocating a claim.
    """
    driverName: 'Optional[str]' = None
    parameters: 'Optional[runtime.RawExtension]' = None





# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import storagemigration_v1alpha1 as m_storagemigration_v1alpha1


class StorageVersionMigrationStatus(res.GlobalSubResource, m_storagemigration_v1alpha1.StorageVersionMigration):
    """* **Extends**: ``models.storagemigration_v1alpha1.StorageVersionMigration``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('storagemigration.k8s.io', 'v1alpha1', 'StorageVersionMigration'),
        parent=res.ResourceDef('storagemigration.k8s.io', 'v1alpha1', 'StorageVersionMigration'),
        plural='storageversionmigrations',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class StorageVersionMigration(res.GlobalResource, m_storagemigration_v1alpha1.StorageVersionMigration):
    """* **Extends**: ``models.storagemigration_v1alpha1.StorageVersionMigration``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``StorageVersionMigrationStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('storagemigration.k8s.io', 'v1alpha1', 'StorageVersionMigration'),
        plural='storageversionmigrations',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = StorageVersionMigrationStatus




# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1


@dataclass
class VolumeAttributesClass(DictMixin):
    """VolumeAttributesClass represents a specification of mutable volume attributes
      defined by the CSI driver. The class can be specified during dynamic
      provisioning of PersistentVolumeClaims, and changed in the
      PersistentVolumeClaim spec after provisioning.

      **parameters**

      * **driverName** ``str`` - Name of the CSI driver This field is immutable.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **parameters** ``Optional[dict]`` - parameters hold volume attributes defined by the CSI driver. These values are
        opaque to the Kubernetes and are passed directly to the CSI driver. The
        underlying storage provider supports changing these attributes on an existing
        volume, however the parameters field itself is immutable. To invoke a volume
        update, a new VolumeAttributesClass should be created with new parameters, and
        the PersistentVolumeClaim should be updated to reference the new
        VolumeAttributesClass.
        This field is required and must contain at least one key/value pair. The keys
        cannot be empty, and the maximum number of parameters is 512, with a
        cumulative max size of 256K. If the CSI driver rejects invalid parameters, the
        target PersistentVolumeClaim will be set to an "Infeasible" state in the
        modifyVolumeStatus field.
    """
    driverName: 'str'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    parameters: 'Optional[dict]' = None


@dataclass
class VolumeAttributesClassList(DictMixin):
    """VolumeAttributesClassList is a collection of VolumeAttributesClass objects.

      **parameters**

      * **items** ``List[VolumeAttributesClass]`` - items is the list of VolumeAttributesClass objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[VolumeAttributesClass]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import core_v1
from . import meta_v1


@dataclass
class CronJob(DictMixin):
    """CronJob represents the configuration of a single cron job.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[CronJobSpec]`` - Specification of the desired behavior of a cron job, including the schedule.
        More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
      * **status** ``Optional[CronJobStatus]`` - Current status of a cron job. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[CronJobSpec]' = None
    status: 'Optional[CronJobStatus]' = None


@dataclass
class CronJobList(DictMixin):
    """CronJobList is a collection of cron jobs.

      **parameters**

      * **items** ``List[CronJob]`` - items is the list of CronJobs.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[CronJob]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class CronJobSpec(DictMixin):
    """CronJobSpec describes how the job execution will look like and when it will
      actually run.

      **parameters**

      * **jobTemplate** ``JobTemplateSpec`` - Specifies the job that will be created when executing a CronJob.
      * **schedule** ``str`` - The schedule in Cron format, see https://en.wikipedia.org/wiki/Cron.
      * **concurrencyPolicy** ``Optional[str]`` - Specifies how to treat concurrent executions of a Job. Valid values are:
        - "Allow" (default): allows CronJobs to run concurrently; - "Forbid": forbids
        concurrent runs, skipping next run if previous run hasn't finished yet; -
        "Replace": cancels currently running job and replaces it with a new one
      * **failedJobsHistoryLimit** ``Optional[int]`` - The number of failed finished jobs to retain. Value must be non-negative
        integer. Defaults to 1.
      * **startingDeadlineSeconds** ``Optional[int]`` - Optional deadline in seconds for starting the job if it misses scheduled time
        for any reason.  Missed jobs executions will be counted as failed ones.
      * **successfulJobsHistoryLimit** ``Optional[int]`` - The number of successful finished jobs to retain. Value must be non-negative
        integer. Defaults to 3.
      * **suspend** ``Optional[bool]`` - This flag tells the controller to suspend subsequent executions, it does not
        apply to already started executions.  Defaults to false.
      * **timeZone** ``Optional[str]`` - The time zone name for the given schedule, see
        https://en.wikipedia.org/wiki/List_of_tz_database_time_zones. If not
        specified, this will default to the time zone of the kube-controller-manager
        process. The set of valid time zone names and the time zone offset is loaded
        from the system-wide time zone database by the API server during CronJob
        validation and the controller manager during execution. If no system-wide time
        zone database can be found a bundled version of the database is used instead.
        If the time zone name becomes invalid during the lifetime of a CronJob or due
        to a change in host configuration, the controller will stop creating new new
        Jobs and will create a system event with the reason UnknownTimeZone. More
        information can be found in
        https://kubernetes.io/docs/concepts/workloads/controllers/cron-jobs/#time-zones
    """
    jobTemplate: 'JobTemplateSpec'
    schedule: 'str'
    concurrencyPolicy: 'Optional[str]' = None
    failedJobsHistoryLimit: 'Optional[int]' = None
    startingDeadlineSeconds: 'Optional[int]' = None
    successfulJobsHistoryLimit: 'Optional[int]' = None
    suspend: 'Optional[bool]' = None
    timeZone: 'Optional[str]' = None


@dataclass
class CronJobStatus(DictMixin):
    """CronJobStatus represents the current state of a cron job.

      **parameters**

      * **active** ``Optional[List[core_v1.ObjectReference]]`` - A list of pointers to currently running jobs.
      * **lastScheduleTime** ``Optional[meta_v1.Time]`` - Information when was the last time the job was successfully scheduled.
      * **lastSuccessfulTime** ``Optional[meta_v1.Time]`` - Information when was the last time the job successfully completed.
    """
    active: 'Optional[List[core_v1.ObjectReference]]' = None
    lastScheduleTime: 'Optional[meta_v1.Time]' = None
    lastSuccessfulTime: 'Optional[meta_v1.Time]' = None


@dataclass
class Job(DictMixin):
    """Job represents the configuration of a single job.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[JobSpec]`` - Specification of the desired behavior of a job. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
      * **status** ``Optional[JobStatus]`` - Current status of a job. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[JobSpec]' = None
    status: 'Optional[JobStatus]' = None


@dataclass
class JobCondition(DictMixin):
    """JobCondition describes current state of a job.

      **parameters**

      * **status** ``str`` - Status of the condition, one of True, False, Unknown.
      * **type** ``str`` - Type of job condition, Complete or Failed.
      * **lastProbeTime** ``Optional[meta_v1.Time]`` - Last time the condition was checked.
      * **lastTransitionTime** ``Optional[meta_v1.Time]`` - Last time the condition transit from one status to another.
      * **message** ``Optional[str]`` - Human readable message indicating details about last transition.
      * **reason** ``Optional[str]`` - (brief) reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastProbeTime: 'Optional[meta_v1.Time]' = None
    lastTransitionTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class JobList(DictMixin):
    """JobList is a collection of jobs.

      **parameters**

      * **items** ``List[Job]`` - items is the list of Jobs.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[Job]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class JobSpec(DictMixin):
    """JobSpec describes how the job execution will look like.

      **parameters**

      * **template** ``core_v1.PodTemplateSpec`` - Describes the pod that will be created when executing a job. The only allowed
        template.spec.restartPolicy values are "Never" or "OnFailure". More info:
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/
      * **activeDeadlineSeconds** ``Optional[int]`` - Specifies the duration in seconds relative to the startTime that the job may
        be continuously active before the system tries to terminate it; value must be
        positive integer. If a Job is suspended (at creation or through an update),
        this timer will effectively be stopped and reset when the Job is resumed
        again.
      * **backoffLimit** ``Optional[int]`` - Specifies the number of retries before marking this job failed. Defaults to 6
      * **backoffLimitPerIndex** ``Optional[int]`` - Specifies the limit for the number of retries within an index before marking
        this index as failed. When enabled the number of failures per index is kept in
        the pod's batch.kubernetes.io/job-index-failure-count annotation. It can only
        be set when Job's completionMode=Indexed, and the Pod's restart policy is
        Never. The field is immutable. This field is beta-level. It can be used when
        the `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default).
      * **completionMode** ``Optional[str]`` - completionMode specifies how Pod completions are tracked. It can be
        `NonIndexed` (default) or `Indexed`.
        `NonIndexed` means that the Job is considered complete when there have been
        .spec.completions successfully completed Pods. Each Pod completion is
        homologous to each other.
        `Indexed` means that the Pods of a Job get an associated completion index from
        0 to (.spec.completions - 1), available in the annotation
        batch.kubernetes.io/job-completion-index. The Job is considered complete when
        there is one successfully completed Pod for each index. When value is
        `Indexed`, .spec.completions must be specified and `.spec.parallelism` must be
        less than or equal to 10^5. In addition, The Pod name takes the form
        `$(job-name)-$(index)-$(random-string)`, the Pod hostname takes the form
        `$(job-name)-$(index)`.
        More completion modes can be added in the future. If the Job controller
        observes a mode that it doesn't recognize, which is possible during upgrades
        due to version skew, the controller skips updates for the Job.
      * **completions** ``Optional[int]`` - Specifies the desired number of successfully finished pods the job should be
        run with.  Setting to null means that the success of any pod signals the
        success of all pods, and allows parallelism to have any positive value.
        Setting to 1 means that parallelism is limited to 1 and the success of that
        pod signals the success of the job. More info:
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/
      * **managedBy** ``Optional[str]`` - ManagedBy field indicates the controller that manages a Job. The k8s Job
        controller reconciles jobs which don't have this field at all or the field
        value is the reserved string `kubernetes.io/job-controller`, but skips
        reconciling Jobs with a custom value for this field. The value must be a valid
        domain-prefixed path (e.g. acme.io/foo) - all characters before the first "/"
        must be a valid subdomain as defined by RFC 1123. All characters trailing the
        first "/" must be valid HTTP Path characters as defined by RFC 3986. The value
        cannot exceed 64 characters.
        This field is alpha-level. The job controller accepts setting the field when
        the feature gate JobManagedBy is enabled (disabled by default).
      * **manualSelector** ``Optional[bool]`` - manualSelector controls generation of pod labels and pod selectors. Leave
        `manualSelector` unset unless you are certain what you are doing. When false
        or unset, the system pick labels unique to this job and appends those labels
        to the pod template.  When true, the user is responsible for picking unique
        labels and specifying the selector.  Failure to pick a unique label may cause
        this and other jobs to not function correctly.  However, You may see
        `manualSelector=true` in jobs that were created with the old
        `extensions/v1beta1` API. More info:
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/#specifying-your-own-pod-selector
      * **maxFailedIndexes** ``Optional[int]`` - Specifies the maximal number of failed indexes before marking the Job as
        failed, when backoffLimitPerIndex is set. Once the number of failed indexes
        exceeds this number the entire Job is marked as Failed and its execution is
        terminated. When left as null the job continues execution of all of its
        indexes and is marked with the `Complete` Job condition. It can only be
        specified when backoffLimitPerIndex is set. It can be null or up to
        completions. It is required and must be less than or equal to 10^4 when is
        completions greater than 10^5. This field is beta-level. It can be used when
        the `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default).
      * **parallelism** ``Optional[int]`` - Specifies the maximum desired number of pods the job should run at any given
        time. The actual number of pods running in steady state will be less than this
        number when ((.spec.completions - .status.successful) < .spec.parallelism),
        i.e. when the work left to do is less than max parallelism. More info:
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/
      * **podFailurePolicy** ``Optional[PodFailurePolicy]`` - Specifies the policy of handling failed pods. In particular, it allows to
        specify the set of actions and conditions which need to be satisfied to take
        the associated action. If empty, the default behaviour applies - the counter
        of failed pods, represented by the jobs's .status.failed field, is incremented
        and it is checked against the backoffLimit. This field cannot be used in
        combination with restartPolicy=OnFailure.
        This field is beta-level. It can be used when the `JobPodFailurePolicy`
        feature gate is enabled (enabled by default).
      * **podReplacementPolicy** ``Optional[str]`` - podReplacementPolicy specifies when to create replacement Pods. Possible
        values are: - TerminatingOrFailed means that we recreate pods
          when they are terminating (has a metadata.deletionTimestamp) or failed.
        - Failed means to wait until a previously created Pod is fully terminated (has
        phase
          Failed or Succeeded) before creating a replacement Pod.
        When using podFailurePolicy, Failed is the the only allowed value.
        TerminatingOrFailed and Failed are allowed values when podFailurePolicy is not
        in use. This is an beta field. To use this, enable the JobPodReplacementPolicy
        feature toggle. This is on by default.
      * **selector** ``Optional[meta_v1.LabelSelector]`` - A label query over pods that should match the pod count. Normally, the system
        sets this field for you. More info:
        https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors
      * **successPolicy** ``Optional[SuccessPolicy]`` - successPolicy specifies the policy when the Job can be declared as succeeded.
        If empty, the default behavior applies - the Job is declared as succeeded only
        when the number of succeeded pods equals to the completions. When the field is
        specified, it must be immutable and works only for the Indexed Jobs. Once the
        Job meets the SuccessPolicy, the lingering pods are terminated.
        This field  is alpha-level. To use this field, you must enable the
        `JobSuccessPolicy` feature gate (disabled by default).
      * **suspend** ``Optional[bool]`` - suspend specifies whether the Job controller should create Pods or not. If a
        Job is created with suspend set to true, no Pods are created by the Job
        controller. If a Job is suspended after creation (i.e. the flag goes from
        false to true), the Job controller will delete all active Pods associated with
        this Job. Users must design their workload to gracefully handle this.
        Suspending a Job will reset the StartTime field of the Job, effectively
        resetting the ActiveDeadlineSeconds timer too. Defaults to false.
      * **ttlSecondsAfterFinished** ``Optional[int]`` - ttlSecondsAfterFinished limits the lifetime of a Job that has finished
        execution (either Complete or Failed). If this field is set,
        ttlSecondsAfterFinished after the Job finishes, it is eligible to be
        automatically deleted. When the Job is being deleted, its lifecycle guarantees
        (e.g. finalizers) will be honored. If this field is unset, the Job won't be
        automatically deleted. If this field is set to zero, the Job becomes eligible
        to be deleted immediately after it finishes.
    """
    template: 'core_v1.PodTemplateSpec'
    activeDeadlineSeconds: 'Optional[int]' = None
    backoffLimit: 'Optional[int]' = None
    backoffLimitPerIndex: 'Optional[int]' = None
    completionMode: 'Optional[str]' = None
    completions: 'Optional[int]' = None
    managedBy: 'Optional[str]' = None
    manualSelector: 'Optional[bool]' = None
    maxFailedIndexes: 'Optional[int]' = None
    parallelism: 'Optional[int]' = None
    podFailurePolicy: 'Optional[PodFailurePolicy]' = None
    podReplacementPolicy: 'Optional[str]' = None
    selector: 'Optional[meta_v1.LabelSelector]' = None
    successPolicy: 'Optional[SuccessPolicy]' = None
    suspend: 'Optional[bool]' = None
    ttlSecondsAfterFinished: 'Optional[int]' = None


@dataclass
class JobStatus(DictMixin):
    """JobStatus represents the current state of a Job.

      **parameters**

      * **active** ``Optional[int]`` - The number of pending and running pods which are not terminating (without a
        deletionTimestamp). The value is zero for finished jobs.
      * **completedIndexes** ``Optional[str]`` - completedIndexes holds the completed indexes when .spec.completionMode =
        "Indexed" in a text format. The indexes are represented as decimal integers
        separated by commas. The numbers are listed in increasing order. Three or more
        consecutive numbers are compressed and represented by the first and last
        element of the series, separated by a hyphen. For example, if the completed
        indexes are 1, 3, 4, 5 and 7, they are represented as "1,3-5,7".
      * **completionTime** ``Optional[meta_v1.Time]`` - Represents time when the job was completed. It is not guaranteed to be set in
        happens-before order across separate operations. It is represented in RFC3339
        form and is in UTC. The completion time is set when the job finishes
        successfully, and only then. The value cannot be updated or removed. The value
        indicates the same or later point in time as the startTime field.
      * **conditions** ``Optional[List[JobCondition]]`` - The latest available observations of an object's current state. When a Job
        fails, one of the conditions will have type "Failed" and status true. When a
        Job is suspended, one of the conditions will have type "Suspended" and status
        true; when the Job is resumed, the status of this condition will become false.
        When a Job is completed, one of the conditions will have type "Complete" and
        status true.
        A job is considered finished when it is in a terminal condition, either
        "Complete" or "Failed". A Job cannot have both the "Complete" and "Failed"
        conditions. Additionally, it cannot be in the "Complete" and "FailureTarget"
        conditions. The "Complete", "Failed" and "FailureTarget" conditions cannot be
        disabled.
        More info:
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/
      * **failed** ``Optional[int]`` - The number of pods which reached phase Failed. The value increases
        monotonically.
      * **failedIndexes** ``Optional[str]`` - FailedIndexes holds the failed indexes when spec.backoffLimitPerIndex is set.
        The indexes are represented in the text format analogous as for the
        `completedIndexes` field, ie. they are kept as decimal integers separated by
        commas. The numbers are listed in increasing order. Three or more consecutive
        numbers are compressed and represented by the first and last element of the
        series, separated by a hyphen. For example, if the failed indexes are 1, 3, 4,
        5 and 7, they are represented as "1,3-5,7". The set of failed indexes cannot
        overlap with the set of completed indexes.
        This field is beta-level. It can be used when the `JobBackoffLimitPerIndex`
        feature gate is enabled (enabled by default).
      * **ready** ``Optional[int]`` - The number of pods which have a Ready condition.
      * **startTime** ``Optional[meta_v1.Time]`` - Represents time when the job controller started processing a job. When a Job
        is created in the suspended state, this field is not set until the first time
        it is resumed. This field is reset every time a Job is resumed from
        suspension. It is represented in RFC3339 form and is in UTC.
        Once set, the field can only be removed when the job is suspended. The field
        cannot be modified while the job is unsuspended or finished.
      * **succeeded** ``Optional[int]`` - The number of pods which reached phase Succeeded. The value increases
        monotonically for a given spec. However, it may decrease in reaction to scale
        down of elastic indexed jobs.
      * **terminating** ``Optional[int]`` - The number of pods which are terminating (in phase Pending or Running and have
        a deletionTimestamp).
        This field is beta-level. The job controller populates the field when the
        feature gate JobPodReplacementPolicy is enabled (enabled by default).
      * **uncountedTerminatedPods** ``Optional[UncountedTerminatedPods]`` - uncountedTerminatedPods holds the UIDs of Pods that have terminated but the
        job controller hasn't yet accounted for in the status counters.
        The job controller creates pods with a finalizer. When a pod terminates
        (succeeded or failed), the controller does three steps to account for it in
        the job status:
        1. Add the pod UID to the arrays in this field. 2. Remove the pod finalizer.
        3. Remove the pod UID from the arrays while increasing the corresponding
            counter.
        Old jobs might not be tracked using this field, in which case the field
        remains null. The structure is empty for finished jobs.
    """
    active: 'Optional[int]' = None
    completedIndexes: 'Optional[str]' = None
    completionTime: 'Optional[meta_v1.Time]' = None
    conditions: 'Optional[List[JobCondition]]' = None
    failed: 'Optional[int]' = None
    failedIndexes: 'Optional[str]' = None
    ready: 'Optional[int]' = None
    startTime: 'Optional[meta_v1.Time]' = None
    succeeded: 'Optional[int]' = None
    terminating: 'Optional[int]' = None
    uncountedTerminatedPods: 'Optional[UncountedTerminatedPods]' = None


@dataclass
class JobTemplateSpec(DictMixin):
    """JobTemplateSpec describes the data a Job should have when created from a
      template

      **parameters**

      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata of the jobs created from this template. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[JobSpec]`` - Specification of the desired behavior of the job. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[JobSpec]' = None


@dataclass
class PodFailurePolicy(DictMixin):
    """PodFailurePolicy describes how failed pods influence the backoffLimit.

      **parameters**

      * **rules** ``List[PodFailurePolicyRule]`` - A list of pod failure policy rules. The rules are evaluated in order. Once a
        rule matches a Pod failure, the remaining of the rules are ignored. When no
        rule matches the Pod failure, the default handling applies - the counter of
        pod failures is incremented and it is checked against the backoffLimit. At
        most 20 elements are allowed.
    """
    rules: 'List[PodFailurePolicyRule]'


@dataclass
class PodFailurePolicyOnExitCodesRequirement(DictMixin):
    """PodFailurePolicyOnExitCodesRequirement describes the requirement for handling
      a failed pod based on its container exit codes. In particular, it lookups the
      .state.terminated.exitCode for each app container and init container status,
      represented by the .status.containerStatuses and .status.initContainerStatuses
      fields in the Pod status, respectively. Containers completed with success
      (exit code 0) are excluded from the requirement check.

      **parameters**

      * **operator** ``str`` - Represents the relationship between the container exit code(s) and the
        specified values. Containers completed with success (exit code 0) are excluded
        from the requirement check. Possible values are:
        - In: the requirement is satisfied if at least one container exit code
          (might be multiple if there are multiple containers not restricted
          by the 'containerName' field) is in the set of specified values.
        - NotIn: the requirement is satisfied if at least one container exit code
          (might be multiple if there are multiple containers not restricted
          by the 'containerName' field) is not in the set of specified values.
        Additional values are considered to be added in the future. Clients should
        react to an unknown operator by assuming the requirement is not satisfied.
      * **values** ``List[int]`` - Specifies the set of values. Each returned container exit code (might be
        multiple in case of multiple containers) is checked against this set of values
        with respect to the operator. The list of values must be ordered and must not
        contain duplicates. Value '0' cannot be used for the In operator. At least one
        element is required. At most 255 elements are allowed.
      * **containerName** ``Optional[str]`` - Restricts the check for exit codes to the container with the specified name.
        When null, the rule applies to all containers. When specified, it should match
        one the container or initContainer names in the pod template.
    """
    operator: 'str'
    values: 'List[int]'
    containerName: 'Optional[str]' = None


@dataclass
class PodFailurePolicyOnPodConditionsPattern(DictMixin):
    """PodFailurePolicyOnPodConditionsPattern describes a pattern for matching an
      actual pod condition type.

      **parameters**

      * **status** ``str`` - Specifies the required Pod condition status. To match a pod condition it is
        required that the specified status equals the pod condition status. Defaults
        to True.
      * **type** ``str`` - Specifies the required Pod condition type. To match a pod condition it is
        required that specified type equals the pod condition type.
    """
    status: 'str'
    type: 'str'


@dataclass
class PodFailurePolicyRule(DictMixin):
    """PodFailurePolicyRule describes how a pod failure is handled when the
      requirements are met. One of onExitCodes and onPodConditions, but not both,
      can be used in each rule.

      **parameters**

      * **action** ``str`` - Specifies the action taken on a pod failure when the requirements are
        satisfied. Possible values are:
        - FailJob: indicates that the pod's job is marked as Failed and all
          running pods are terminated.
        - FailIndex: indicates that the pod's index is marked as Failed and will
          not be restarted.
          This value is beta-level. It can be used when the
          `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default).
        - Ignore: indicates that the counter towards the .backoffLimit is not
          incremented and a replacement pod is created.
        - Count: indicates that the pod is handled in the default way - the
          counter towards the .backoffLimit is incremented.
        Additional values are considered to be added in the future. Clients should
        react to an unknown action by skipping the rule.
      * **onExitCodes** ``Optional[PodFailurePolicyOnExitCodesRequirement]`` - Represents the requirement on the container exit codes.
      * **onPodConditions** ``Optional[List[PodFailurePolicyOnPodConditionsPattern]]`` - Represents the requirement on the pod conditions. The requirement is
        represented as a list of pod condition patterns. The requirement is satisfied
        if at least one pattern matches an actual pod condition. At most 20 elements
        are allowed.
    """
    action: 'str'
    onExitCodes: 'Optional[PodFailurePolicyOnExitCodesRequirement]' = None
    onPodConditions: 'Optional[List[PodFailurePolicyOnPodConditionsPattern]]' = None


@dataclass
class SuccessPolicy(DictMixin):
    """SuccessPolicy describes when a Job can be declared as succeeded based on the
      success of some indexes.

      **parameters**

      * **rules** ``List[SuccessPolicyRule]`` - rules represents the list of alternative rules for the declaring the Jobs as
        successful before `.status.succeeded >= .spec.completions`. Once any of the
        rules are met, the "SucceededCriteriaMet" condition is added, and the
        lingering pods are removed. The terminal state for such a Job has the
        "Complete" condition. Additionally, these rules are evaluated in order; Once
        the Job meets one of the rules, other rules are ignored. At most 20 elements
        are allowed.
    """
    rules: 'List[SuccessPolicyRule]'


@dataclass
class SuccessPolicyRule(DictMixin):
    """SuccessPolicyRule describes rule for declaring a Job as succeeded. Each rule
      must have at least one of the "succeededIndexes" or "succeededCount"
      specified.

      **parameters**

      * **succeededCount** ``Optional[int]`` - succeededCount specifies the minimal required size of the actual set of the
        succeeded indexes for the Job. When succeededCount is used along with
        succeededIndexes, the check is constrained only to the set of indexes
        specified by succeededIndexes. For example, given that succeededIndexes is
        "1-4", succeededCount is "3", and completed indexes are "1", "3", and "5", the
        Job isn't declared as succeeded because only "1" and "3" indexes are
        considered in that rules. When this field is null, this doesn't default to any
        value and is never evaluated at any time. When specified it needs to be a
        positive integer.
      * **succeededIndexes** ``Optional[str]`` - succeededIndexes specifies the set of indexes which need to be contained in
        the actual set of the succeeded indexes for the Job. The list of indexes must
        be within 0 to ".spec.completions-1" and must not contain duplicates. At least
        one element is required. The indexes are represented as intervals separated by
        commas. The intervals can be a decimal integer or a pair of decimal integers
        separated by a hyphen. The number are listed in represented by the first and
        last element of the series, separated by a hyphen. For example, if the
        completed indexes are 1, 3, 4, 5 and 7, they are represented as "1,3-5,7".
        When this field is null, this field doesn't default to any value and is never
        evaluated at any time.
    """
    succeededCount: 'Optional[int]' = None
    succeededIndexes: 'Optional[str]' = None


@dataclass
class UncountedTerminatedPods(DictMixin):
    """UncountedTerminatedPods holds UIDs of Pods that have terminated but haven't
      been accounted in Job status counters.

      **parameters**

      * **failed** ``Optional[List[str]]`` - failed holds UIDs of failed Pods.
      * **succeeded** ``Optional[List[str]]`` - succeeded holds UIDs of succeeded Pods.
    """
    failed: 'Optional[List[str]]' = None
    succeeded: 'Optional[List[str]]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1


@dataclass
class GroupVersionResource(DictMixin):
    """The names of the group, the version, and the resource.

      **parameters**

      * **group** ``Optional[str]`` - The name of the group.
      * **resource** ``Optional[str]`` - The name of the resource.
      * **version** ``Optional[str]`` - The name of the version.
    """
    group: 'Optional[str]' = None
    resource: 'Optional[str]' = None
    version: 'Optional[str]' = None


@dataclass
class MigrationCondition(DictMixin):
    """Describes the state of a migration at a certain point.

      **parameters**

      * **status** ``str`` - Status of the condition, one of True, False, Unknown.
      * **type** ``str`` - Type of the condition.
      * **lastUpdateTime** ``Optional[meta_v1.Time]`` - The last time this condition was updated.
      * **message** ``Optional[str]`` - A human readable message indicating details about the transition.
      * **reason** ``Optional[str]`` - The reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastUpdateTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class StorageVersionMigration(DictMixin):
    """StorageVersionMigration represents a migration of stored data to the latest
      storage version.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[StorageVersionMigrationSpec]`` - Specification of the migration.
      * **status** ``Optional[StorageVersionMigrationStatus]`` - Status of the migration.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[StorageVersionMigrationSpec]' = None
    status: 'Optional[StorageVersionMigrationStatus]' = None


@dataclass
class StorageVersionMigrationList(DictMixin):
    """StorageVersionMigrationList is a collection of storage version migrations.

      **parameters**

      * **items** ``List[StorageVersionMigration]`` - Items is the list of StorageVersionMigration
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[StorageVersionMigration]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class StorageVersionMigrationSpec(DictMixin):
    """Spec of the storage version migration.

      **parameters**

      * **resource** ``GroupVersionResource`` - The resource that is being migrated. The migrator sends requests to the
        endpoint serving the resource. Immutable.
      * **continueToken** ``Optional[str]`` - The token used in the list options to get the next chunk of objects to
        migrate. When the .status.conditions indicates the migration is "Running",
        users can use this token to check the progress of the migration.
    """
    resource: 'GroupVersionResource'
    continueToken: 'Optional[str]' = None


@dataclass
class StorageVersionMigrationStatus(DictMixin):
    """Status of the storage version migration.

      **parameters**

      * **conditions** ``Optional[List[MigrationCondition]]`` - The latest available observations of the migration's current state.
      * **resourceVersion** ``Optional[str]`` - ResourceVersion to compare with the GC cache for performing the migration.
        This is the current resource version of given group, version and resource when
        kube-controller-manager first observes this StorageVersionMigration resource.
    """
    conditions: 'Optional[List[MigrationCondition]]' = None
    resourceVersion: 'Optional[str]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1


@dataclass
class IPAddress(DictMixin):
    """IPAddress represents a single IP of a single IP Family. The object is designed
      to be used by APIs that operate on IP addresses. The object is used by the
      Service core API for allocation of IP addresses. An IP address can be
      represented in different formats, to guarantee the uniqueness of the IP, the
      name of the object is the IP address in canonical format, four decimal digits
      separated by dots suppressing leading zeros for IPv4 and the representation
      defined by RFC 5952 for IPv6. Valid: 192.168.1.5 or 2001:db8::1 or
      2001:db8:aaaa:bbbb:cccc:dddd:eeee:1 Invalid: 10.01.2.3 or 2001:db8:0:0:0::1

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[IPAddressSpec]`` - spec is the desired state of the IPAddress. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[IPAddressSpec]' = None


@dataclass
class IPAddressList(DictMixin):
    """IPAddressList contains a list of IPAddress.

      **parameters**

      * **items** ``List[IPAddress]`` - items is the list of IPAddresses.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[IPAddress]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class IPAddressSpec(DictMixin):
    """IPAddressSpec describe the attributes in an IP Address.

      **parameters**

      * **parentRef** ``ParentReference`` - ParentRef references the resource that an IPAddress is attached to. An
        IPAddress must reference a parent object.
    """
    parentRef: 'ParentReference'


@dataclass
class ParentReference(DictMixin):
    """ParentReference describes a reference to a parent object.

      **parameters**

      * **name** ``str`` - Name is the name of the object being referenced.
      * **resource** ``str`` - Resource is the resource of the object being referenced.
      * **group** ``Optional[str]`` - Group is the group of the object being referenced.
      * **namespace** ``Optional[str]`` - Namespace is the namespace of the object being referenced.
    """
    name: 'str'
    resource: 'str'
    group: 'Optional[str]' = None
    namespace: 'Optional[str]' = None


@dataclass
class ServiceCIDR(DictMixin):
    """ServiceCIDR defines a range of IP addresses using CIDR format (e.g.
      192.168.0.0/24 or 2001:db2::/64). This range is used to allocate ClusterIPs to
      Service objects.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[ServiceCIDRSpec]`` - spec is the desired state of the ServiceCIDR. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
      * **status** ``Optional[ServiceCIDRStatus]`` - status represents the current state of the ServiceCIDR. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ServiceCIDRSpec]' = None
    status: 'Optional[ServiceCIDRStatus]' = None


@dataclass
class ServiceCIDRList(DictMixin):
    """ServiceCIDRList contains a list of ServiceCIDR objects.

      **parameters**

      * **items** ``List[ServiceCIDR]`` - items is the list of ServiceCIDRs.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[ServiceCIDR]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ServiceCIDRSpec(DictMixin):
    """ServiceCIDRSpec define the CIDRs the user wants to use for allocating
      ClusterIPs for Services.

      **parameters**

      * **cidrs** ``Optional[List[str]]`` - CIDRs defines the IP blocks in CIDR notation (e.g. "192.168.0.0/24" or
        "2001:db8::/64") from which to assign service cluster IPs. Max of two CIDRs is
        allowed, one of each IP family. This field is immutable.
    """
    cidrs: 'Optional[List[str]]' = None


@dataclass
class ServiceCIDRStatus(DictMixin):
    """ServiceCIDRStatus describes the current state of the ServiceCIDR.

      **parameters**

      * **conditions** ``Optional[List[meta_v1.Condition]]`` - conditions holds an array of metav1.Condition that describe the state of the
        ServiceCIDR. Current service state
    """
    conditions: 'Optional[List[meta_v1.Condition]]' = None


